import os.path
from typing import List, Optional

from aiogrpcclient import BaseGrpcClient
from summa.proto import consumer_service_pb2 as consumer_service_pb
from summa.proto import index_service_pb2 as index_service_pb
from summa.proto import search_service_pb2 as search_service_pb
from summa.proto.consumer_service_pb2_grpc import ConsumerApiStub
from summa.proto.index_service_pb2_grpc import IndexApiStub
from summa.proto.search_service_pb2_grpc import SearchApiStub


class SummaClient(BaseGrpcClient):
    stub_clses = {
        'consumer_api': ConsumerApiStub,
        'index_api': IndexApiStub,
        'search_api': SearchApiStub,
    }

    async def commit_index(
        self,
        index_name: str,
        request_id: Optional[str] = None,
        session_id: Optional[str] = None,
    ) -> index_service_pb.CommitIndexResponse:
        return await self.stubs['index_api'].commit_index(
            index_service_pb.CommitIndexRequest(
                index_name=index_name,
            ),
            metadata=(('request-id', request_id), ('session-id', session_id)),
        )

    async def create_consumer(
        self,
        consumer_name: str,
        bootstrap_servers: List[str],
        group_id: str,
        index_name: str,
        topics: List[str],
        threads: Optional[int] = None,
        request_id: Optional[str] = None,
        session_id: Optional[str] = None,
    ) -> consumer_service_pb.CreateConsumerResponse:
        return await self.stubs['consumer_api'].create_consumer(
            consumer_service_pb.CreateConsumerRequest(
                consumer_name=consumer_name,
                bootstrap_servers=bootstrap_servers,
                group_id=group_id,
                index_name=index_name,
                topics=topics,
                threads=threads,
            ),
            metadata=(('request-id', request_id), ('session-id', session_id)),
        )

    async def create_index(
        self,
        index_name: str,
        schema: str,
        primary_key: Optional[str] = None,
        default_fields: Optional[List[str]] = None,
        compression: Optional[str] = None,
        writer_heap_size: Optional[int] = None,
        writer_threads: Optional[int] = None,
        autocommit_interval_ms: Optional[int] = None,
        request_id: Optional[str] = None,
        session_id: Optional[str] = None,
    ) -> index_service_pb.CreateIndexResponse:
        if os.path.exists(schema):
            with open(schema, 'r') as f:
                schema = f.read()
        return await self.stubs['index_api'].create_index(
            index_service_pb.CreateIndexRequest(
                index_name=index_name,
                schema=schema,
                primary_key=primary_key,
                default_fields=default_fields,
                compression=compression,
                writer_heap_size=writer_heap_size,
                writer_threads=writer_threads,
                autocommit_interval_ms=autocommit_interval_ms,
            ),
            metadata=(('request-id', request_id), ('session-id', session_id)),
        )

    async def delete_consumer(
        self,
        index_name: str,
        cascade: bool = False,
        delete_topics: bool = False,
        request_id: Optional[str] = None,
        session_id: Optional[str] = None,
    ) -> consumer_service_pb.DeleteConsumerResponse:
        return await self.stubs['consumer_api'].delete_consumer(
            consumer_service_pb.DeleteConsumerRequest(index_name=index_name, cascade=cascade, delete_topics=delete_topics),
            metadata=(('request-id', request_id), ('session-id', session_id)),
        )

    async def delete_index(
        self,
        index_name: str,
        cascade: bool = False,
        request_id: Optional[str] = None,
        session_id: Optional[str] = None,
    ) -> index_service_pb.DeleteIndexResponse:
        return await self.stubs['index_api'].delete_index(
            index_service_pb.DeleteIndexRequest(index_name=index_name, cascade=cascade),
            metadata=(('request-id', request_id), ('session-id', session_id)),
        )

    async def get_consumer(
        self,
        consumer_name: str,
        request_id: Optional[str] = None,
        session_id: Optional[str] = None,
    ) -> consumer_service_pb.GetConsumerResponse:
        return await self.stubs['consumer_api'].get_consumer(
            consumer_service_pb.GetConsumerRequest(consumer_name=consumer_name),
            metadata=(('request-id', request_id), ('session-id', session_id)),
        )

    async def get_consumers(
        self,
        request_id: Optional[str] = None,
        session_id: Optional[str] = None,
    ) -> consumer_service_pb.GetConsumersResponse:
        return await self.stubs['consumer_api'].get_consumers(
            consumer_service_pb.GetConsumersRequest(),
            metadata=(('request-id', request_id), ('session-id', session_id)),
        )

    async def get_index(
        self,
        index_name: str,
        request_id: Optional[str] = None,
        session_id: Optional[str] = None,
    ) -> index_service_pb.GetIndexResponse:
        return await self.stubs['index_api'].get_index(
            index_service_pb.GetIndexRequest(index_name=index_name),
            metadata=(('request-id', request_id), ('session-id', session_id)),
        )

    async def get_indices(
        self,
        request_id: Optional[str] = None,
        session_id: Optional[str] = None,
    ) -> index_service_pb.GetIndicesResponse:
        return await self.stubs['index_api'].get_indices(
            index_service_pb.GetIndicesRequest(),
            metadata=(('request-id', request_id), ('session-id', session_id)),
        )

    async def get_aliases(
        self,
        request_id: Optional[str] = None,
        session_id: Optional[str] = None,
    ) -> index_service_pb.GetIndexAliasesResponse:
        return await self.stubs['index_api'].get_aliases(
            index_service_pb.GetIndexAliasesRequest(),
            metadata=(('request-id', request_id), ('session-id', session_id)),
        )

    async def search(
        self,
        index_alias: str,
        query: str,
        limit: Optional[int] = 10,
        offset: Optional[int] = 0,
        request_id: Optional[str] = None,
        session_id: Optional[str] = None,
    ) -> search_service_pb.SearchResponse:
        return await self.stubs['search_api'].search(
            search_service_pb.SearchRequest(
                index_alias=index_alias,
                query=query,
                limit=limit,
                offset=offset,
            ),
            metadata=(('request-id', request_id), ('session-id', session_id)),
        )

    async def set_alias(
        self,
        index_alias: str,
        index_name: str,
        request_id: Optional[str] = None,
        session_id: Optional[str] = None,
    ) -> index_service_pb.SetIndexAliasResponse:
        return await self.stubs['index_api'].set_alias(
            index_service_pb.SetIndexAliasRequest(index_alias=index_alias, index_name=index_name),
            metadata=(('request-id', request_id), ('session-id', session_id)),
        )
